package de.intevation.flys.exports;

import java.awt.Font;
import java.util.Date;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;

import org.jfree.data.general.SeriesException;
import org.jfree.data.time.Minute;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.Timerange;
import de.intevation.flys.artifacts.model.WQTimerange;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class HistoricalDischargeCurveGenerator
extends      TimeseriesChartGenerator
implements   FacetTypes
{
    private static Logger logger =
        Logger.getLogger(HistoricalDischargeCurveGenerator.class);


    public static enum YAXIS {
        Q(0);
        protected int idx;
        private YAXIS(int c) {
            idx = c;
        }
    }


    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {
            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }


    @Override
    protected String getDefaultChartTitle() {
        return "TODO: CHART TITLE";
    }


    /**
     * Empty (suppress subtitle).
     */
    @Override
    protected void addSubtitles(JFreeChart chart) {
        // this chart has no subtitle
    }


    @Override
    protected String getDefaultXAxisLabel() {
        return "TODO: DEFAULT X AXIS LABEL";
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        return "TODO: DEFAULT Y AXIS LABEL FOR: " + pos;
    }


    /**
     * Creates a new Y-Axis.
     *
     * @param index the index of the Y axis.
     *
     * @return an new NumberAxis with label from <i>getYAxisLabel()</i>.
     */
    @Override
    protected NumberAxis createYAxis(int index) {
        Font labelFont = new Font("Tahoma", Font.BOLD, 14);
        String label   = "default";

        if (index == YAXIS.Q.idx) {
            label = getYAxisLabel(0);
        }

        NumberAxis axis = createNumberAxis(index, label);
        axis.setLabelFont(labelFont);
        axis.setAutoRangeIncludesZero(false);

        return axis;
    }


    public void doOut(
        ArtifactAndFacet artifactFacet,
        Document         theme,
        boolean          visible
    ) {
        String name = artifactFacet.getFacetName();
        logger.debug("HistoricalDischargeCurveGenerator.doOut: " + name);


        if (name.equals(HISTORICAL_DISCHARGE_Q)) {
            doHistoricalDischargeOut(
                (FLYSArtifact) artifactFacet.getArtifact(),
                artifactFacet.getData(context),
                artifactFacet.getFacetDescription(),
                theme,
                visible);
        }
        else {
           logger.warn("doOut(): unknown facet name: " + name);
           return;
        }
    }


    protected void doHistoricalDischargeOut(
        FLYSArtifact artifact,
        Object       data,
        String       desc,
        Document     theme,
        boolean      visible)
    {
        logger.debug("doHistoricalDischargeOut(): description = " + desc);
        logger.warn("TODO IMPLEMENT ME");

        WQTimerange wqt = (WQTimerange) data;

        TimeSeriesCollection tsc = newTimeSeriesCollection(wqt, desc);

        addAxisDataset(tsc, 0, visible);
    }


    /**
     * Creates a new TimeSeriesCollection with a single TimeSeries. The
     * TimeSeries will consist of two RegularTimePeriods for each W/Q value
     * provided by <i>wqt</i>. This has the effect, that the line in the chart
     * looks like a "step chart".
     */
    protected TimeSeriesCollection newTimeSeriesCollection(
        WQTimerange wqt,
        String      desc
    ) {
        logger.debug("Create new TimeSeriesCollection for: " + desc);

        TimeSeriesCollection tsc = new TimeSeriesCollection();
        TimeSeries        series = new TimeSeries(desc);

        Timerange tr = null;
        double[] res = new double[2];

        for (int i = 0, n = wqt.size(); i < n; i++) {
            res = wqt.get(i, res);

            RegularTimePeriod[] rtp = newRegularTimePeriod(wqt.getTimerange(i));

            try {
                // TODO WHICH (W/Q) IS REQUIRED?
                series.add(rtp[0], res[1]);
                series.add(rtp[1], res[1]);
            }
            catch (SeriesException se) {
                logger.warn("Error while adding TimePeriod: " + se);
            }
        }

        tsc.addSeries(series);

        return tsc;
    }


    /**
     * Creates an array that consists of two <i>Minute</i> periods [start, end].
     *
     * @param timerange Supports start and end time.
     *
     * @return an array with two <i>Minute</i> periods [start, end].
     */
    protected RegularTimePeriod[] newRegularTimePeriod(Timerange timerange) {
        Date start = new Date(timerange.getStart());
        Date end   = new Date(timerange.getEnd() - 1000 * 60);

        return new RegularTimePeriod[] {
            new Minute(start),
            new Minute(end)
        };
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
